@@ -19,12 +19,12 @@ typedef enum {
   CC_ALG_SENDME = 0,
 
   /**
-   * Prop#324 TOR_WESTWOOD - Deliberately agressive. Westwood may not even
+   * Prop#324 TOR_WESTWOOD - Deliberately aggressive. Westwood may not even
    * converge to fairness in some cases because max RTT will also increase
-   * on congesgtion, which boosts the Westwood RTT congestion threshhold. So it
+   * on congestion, which boosts the Westwood RTT congestion threshold. So it
    * can cause runaway queue bloat, which may or may not lead to a robot
    * uprising... Ok that's Westworld, not Westwood. Still, we need to test
-   * Vegas and NOLA against something more agressive to ensure they do not
+   * Vegas and NOLA against something more aggressive to ensure they do not
    * starve in the presence of cheaters. We also need to make sure cheaters
    * trigger the oomkiller in those cases.
    */
@@ -32,7 +32,7 @@ typedef enum {
 
   /**
    * Prop#324 TOR_VEGAS - TCP Vegas-style BDP tracker. Because Vegas backs off
-   * whenever it detects queue delay, it can be beaten out by more agressive
+   * whenever it detects queue delay, it can be beaten out by more aggressive
    * algs. However, in live network testing, it seems to do just fine against
    * current SENDMEs. It outperforms Westwood and does not stall. */
   CC_ALG_VEGAS = 2,
@@ -40,9 +40,9 @@ typedef enum {
   /**
    * Prop#324: TOR_NOLA - NOLA looks the BDP right in the eye and uses it
    * immediately as CWND. No slow start, no other congestion signals, no delay,
-   * no bullshit. Like TOR_VEGAS, it also uses agressive BDP estimates, to
-   * avoid out-competition. It seems a bit better throughput than Vegas,
-   * but its agressive BDP and rapid updates may lead to more queue latency. */
+   * no bullshit. Like TOR_VEGAS, it also uses aggressive BDP estimates, to
+   * avoid out-competition. It seems a bit better throughput than Vegas, but
+   * its aggressive BDP and rapid updates may lead to more queue latency. */
   CC_ALG_NOLA = 3,
 } cc_alg_t;
 
@@ -97,6 +97,10 @@ struct westwood_params_t {
 
 /** Vegas algorithm parameters. */
 struct vegas_params_t {
+    /** The slow-start cwnd cap for RFC3742 */
+    uint32_t ss_cwnd_cap;
+    /** The maximum slow-start cwnd */
+    uint32_t ss_cwnd_max;
     /** The queue use allowed before we exit slow start */
     uint16_t gamma;
     /** The queue use below which we increment cwnd */
@@ -147,18 +151,25 @@ struct congestion_control_t {
   /**
    * For steady-state: the number of sendme acks until we will acknowledge
    * a congestion event again. It starts out as the number of sendme acks
-   * in a congestion windowm and is decremented each ack. When this reaches
+   * in a congestion window and is decremented each ack. When this reaches
    * 0, it means we should examine our congestion algorithm conditions.
    * In this way, we only react to one congestion event per congestion window.
    *
    * It is also reset to 0 immediately whenever the circuit's orconn is
    * blocked, and when a previously blocked orconn is unblocked.
    */
-  uint64_t next_cc_event;
+  uint16_t next_cc_event;
+
+  /** Counts down until we process a cwnd worth of SENDME acks.
+   * Used to track full cwnd status. */
+  uint16_t next_cwnd_event;
 
   /** Are we in slow start? */
   bool in_slow_start;
 
+  /** Has the cwnd become full since last cwnd update? */
+  bool cwnd_full;
+
   /** Is the local channel blocked on us? That's a congestion signal */
   bool blocked_chan;
 
@@ -193,7 +204,7 @@ struct congestion_control_t {
   bdp_alg_t bdp_alg;
 
   /** Algorithm-specific parameters. The specific struct that is used
-   * depends upon the algoritghm selected by the cc_alg parameter.
+   * depends upon the algorithm selected by the cc_alg parameter.
    * These should not be accessed anywhere other than the algorithm-specific
    * files. */
   union {
@@ -204,7 +215,7 @@ struct congestion_control_t {
 };
 
 /**
- * Returns the number of sendme acks we will recieve before we update cwnd.
+ * Returns the number of sendme acks we will receive before we update cwnd.
  *
  * Congestion control literature recommends only one update of cwnd per
  * cwnd worth of acks. However, we can also tune this to be more frequent
@@ -220,13 +231,23 @@ static inline uint64_t CWND_UPDATE_RATE(const struct congestion_control_t *cc)
    * of acks */
 
   if (cc->in_slow_start) {
-    return ((cc->cwnd + cc->sendme_inc/2)/cc->sendme_inc);
+    return 1;
   } else {
     return ((cc->cwnd + cc->cwnd_inc_rate*cc->sendme_inc/2)
            / (cc->cwnd_inc_rate*cc->sendme_inc));
   }
 }
 
+/**
+ * Gives us the number of SENDMEs in a CWND, rounded.
+ */
+static inline uint64_t SENDME_PER_CWND(const struct congestion_control_t *cc)
+{
+  /* We add cwnd_inc_rate*sendme_inc/2 to round to nearest integer number
+   * of acks */
+  return ((cc->cwnd + cc->sendme_inc/2)/cc->sendme_inc);
+}
+
 /**
  * Returns the amount to increment the congestion window each update,
  * during slow start.
