@@ -351,7 +351,7 @@ circuit_sendme_cell_is_next(int deliver_window, int sendme_inc)
    *
    * Because deliver_window starts at CIRCWINDOW_START and counts down,
    * to get the actual number of received cells for this check, we must
-   * first convert to receieved cells, or the modulus operator will fail.
+   * first convert to received cells, or the modulus operator will fail.
    */
   tor_assert(deliver_window <= CIRCWINDOW_START);
   if (((CIRCWINDOW_START - (deliver_window - 1)) % sendme_inc) != 0) {
@@ -474,12 +474,14 @@ sendme_process_circuit_level(crypt_path_t *layer_hint,
     return -END_CIRC_REASON_TORPROTOCOL;
   }
 
+  /* origin circuits need to count valid sendmes as valid protocol data */
+  if (CIRCUIT_IS_ORIGIN(circ)) {
+    circuit_read_valid_data(TO_ORIGIN_CIRCUIT(circ), cell_payload_len);
+  }
+
   // Get CC
   if (layer_hint) {
     cc = layer_hint->ccontrol;
-
-    /* origin circuits need to count valid sendmes as valid protocol data */
-    circuit_read_valid_data(TO_ORIGIN_CIRCUIT(circ), cell_payload_len);
   } else {
     cc = circ->ccontrol;
   }
