@@ -55,7 +55,7 @@ static const char *str_ed25519_basepoint =
 
 #ifdef HAVE_SYS_UN_H
 
-/** Given <b>ports</b>, a smarlist containing hs_port_config_t,
+/** Given <b>ports</b>, a smartlist containing hs_port_config_t,
  * add the given <b>p</b>, a AF_UNIX port to the list. Return 0 on success
  * else return -ENOSYS if AF_UNIX is not supported (see function in the
  * #else statement below). */
@@ -432,7 +432,7 @@ get_second_cached_disaster_srv(void)
  * thus will be ignored for the param construction.
  *
  * The result is put in param_out. */
-static void
+STATIC void
 build_blinded_key_param(const ed25519_public_key_t *pubkey,
                         const uint8_t *secret, size_t secret_len,
                         uint64_t period_num, uint64_t period_length,
@@ -1687,7 +1687,7 @@ hs_get_extend_info_from_lspecs(const smartlist_t *lspecs,
   /* We do have everything for which we think we can connect successfully. */
   info = extend_info_new(NULL, legacy_id,
                          (have_ed25519_id) ? &ed25519_pk : NULL, NULL,
-                         onion_key, &ap.addr, ap.port);
+                         onion_key, &ap.addr, ap.port, NULL, false);
  done:
   return info;
 }
